%% INTRODUCTION %%
% In this code, we are doing two things. First, we are maximizing Buffered AUC
% for an arifical binary classification problem where we have a linear loss function
% and the loss is the difference w^T(X_n - X_p)
% where w is the vector of paramters and X_n is a matrix of samples from the positive label
% class and X_p is the matrix of samples from the negative label class. 
% This is equivalent to minimizing the expectation E[max{w^T(X_n - X_p)+1},0}] which is the partial moment
% with paramter -1
% 
% The second thing we are doing is simply calculating bAUC=1-bPOE for the set of losses at the optimal solution.
%This is meant to demosntrate the procedure to calucalte bAUC for a set of ranking losses.

%We perform the classification and calculation over ten runs and average
%the result, reporting the average and standard deviation. In each run, we
%generate a random matrix of positive samples and negative samples.


%For more details about using PSG in MATLAB, documentation can be found at
%http://www.aorda.com/html/PSG_Help_HTML/index.html?why_psg.htm


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initialize the sample sizes
%num_n=number of negative labeled samples
%num_p=number of positive labeled samples
num_n=100000
num_p=100000

%We use t_opt to time the execution of the optimization procedure
%We use t_calc to time the execution of the calculation procedure
t_opt=zeros(10,1);
t_calc=zeros(10,1);

%We run the optimization procedure and log the mean and standard deviation
%of run time with variables AVG_opt and STD_opt, which are at the bottom of
%this code page.


for i=1:10
    disp(i)
    rng(i)
    
    %To Maximize bAUC, we begin by inputing two data matrices
    %First, we have a random matrix of -1 class points with features
    %drawn uniformly from the interval [-.25,.75]
    dimension=10;
    matrix_n=rand(num_n,dimension)-.25;
    
    
    %Second, we have the matrix of +1 class points, with features drawn
    %uniformly fromt he interval [0,1]
    matrix_p=rand(num_p,dimension);
  
    
    %We start the timer with 'tic' to time the optimization procedure
    tic
    %We need to also pack the matrix object for PSG to process
    iargstruc_arr(1) = tbpsg_matrix_pack('matrix_n', matrix_n);
    iargstruc_arr(2) = tbpsg_matrix_pack('matrix_p', matrix_p);

    parameter_alpha_data=-1; %set parameter for the partial moment function
    iargstruc_arr(3) = tbpsg_parameter_pack('parameter_alpha', parameter_alpha_data);
    
    %set the problem statment to minimize 1-bAUC, i.e. to minimize the
    %partial moment
    problem_statement = sprintf('%s\n',...
    'minimize',...
    '  pm_pen(parameter_alpha,L(matrix_n)-L(matrix_p))',...
    ' ');
    
    %Run the PSG optimization routine using using tbpsg_run function:
    [solution_str, outargstruc_arr] = tbpsg_run(problem_statement, iargstruc_arr);

    
    %Retrieve the optimal hyperplane
    w = tbpsg_optimal_point_data(solution_str, outargstruc_arr);
    vars = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);
    
    %stop timer for optimization
    t_opt(i)=toc
    
    
    
    
    
    
        
    %Calculate the scores at the optimal solution
    scores_n=matrix_n*w.';
    scores_p=matrix_p*w.';
    
    
    %We start the timer with 'tic' to time the calculation 
    tic
    %To calculate bAUC we need to solve the one dimension partial moment
    %problem min_(a>=0) E[max{a( scores_n-scores_p) +1,0} ]
    
    %We first pack the data into structures for PSG to process
    iargstruc_arr(1) = tbpsg_matrix_pack('matrix_n', scores_n, [], [], []);
    iargstruc_arr(2) = tbpsg_matrix_pack('matrix_p', scores_p, [],  [], []);
    parameter_alpha_data=-1;
    iargstruc_arr(3) = tbpsg_parameter_pack('parameter_alpha', parameter_alpha_data);
    
    %We form the problem statement, which is min_(a>=0) E[max{a( scores_n-scores_p) +1,0} ]
        problem_statement = sprintf('%s\n',...
    'minimize',...
    '  pm_pen(parameter_alpha,L(matrix_n)-L(matrix_p))',...
    'Box: >=0 ',...
    ' ');

    %Optimize problem using tbpsg_run function:
    [solution_str, outargstruc_arr] = tbpsg_run(problem_statement, iargstruc_arr);
    %print solution
    disp(solution_str)
    %stop timing of calculation
    t_calc(i)=toc
end

AVG_Opt=mean(t_opt)
STD_Opt=std(t_opt)

AVG_Calc=mean(t_calc)
STD_Calc=std(t_calc)